==> carpet-addons-not-found-1.21.1-2.0.0-sources.jar: carpetaddonsnotfound/mixins/LivingEntity_DropAllXpOnPlayerDeathMixin.java <==
package carpetaddonsnotfound.mixins;

import carpetaddonsnotfound.CarpetAddonsNotFoundSettings;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_3218;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Redirect;

@Mixin(class_1309.class)
public abstract class LivingEntity_DropAllXpOnPlayerDeathMixin {
  @Redirect(
          //#if MC>12103
          //$$ method = "dropExperience",
          //#else
          method = "dropXp",
          //#endif
          at = @At(
                  value = "INVOKE",
                  //#if MC>12103
                  //$$ target = "Lnet/minecraft/entity/LivingEntity;getExperienceToDrop(Lnet/minecraft/server/world/ServerWorld;Lnet/minecraft/entity/Entity;)I"
                  //#elseif MC>12006
                  target = "Lnet/minecraft/entity/LivingEntity;getXpToDrop(Lnet/minecraft/server/world/ServerWorld;Lnet/minecraft/entity/Entity;)I"
                  //#elseif MC>11802
                  //$$ target = "Lnet/minecraft/entity/LivingEntity;getXpToDrop()I"
                  //#else
                  //$$ target = "Lnet/minecraft/entity/LivingEntity;getXpToDrop(Lnet/minecraft/entity/player/PlayerEntity;)I"
                  //#endif
  ))
  private int onDropXp(
          //#if MC>12006
          class_1309 instance,
          class_3218 world,
          class_1297 attacker
          //#elseif MC>11802
          //$$ LivingEntity instance
          //#else
          //$$ LivingEntity instance,
          //$$ PlayerEntity attackingPlayer
          //#endif
                      ) {
    if (!CarpetAddonsNotFoundSettings.dropAllXpOnPlayerDeath ||
        !((class_1309) instance instanceof class_1657 player)) {
      //#if MC>12103
      //$$ return instance.getExperienceToDrop(world, attacker);
      //#elseif MC>12006
      return instance.method_59923(world, attacker);
      //#elseif MC>11802
      //$$ return instance.getXpToDrop();
      //#else
      //$$ return ((LivingEntityInvokerMixin) instance).invokeGetXpToDrop(attackingPlayer);
      //#endif
    }

    double totalExperience = calculateTotalExperience(player);
    return (int)Math.ceil(totalExperience);
  }

  // This exists because just using the totalExperience field doesn't work if levels have been set separately to xp points
  @Unique
  private double calculateTotalExperience(class_1657 player) {
    double xpLevel = player.field_7520 + player.field_7510;
    if (xpLevel < 17) {
      return Math.pow(xpLevel, 2) + 6 * xpLevel;
    }

    if (xpLevel < 32) {
      return 2.5 * Math.pow(xpLevel, 2) - 40.5 * xpLevel + 360;
    }

    return 4.5 * Math.pow(xpLevel, 2) - 162.5 * xpLevel + 2220;
  }
}

==> carpet-addons-not-found-1.21.5-2.0.0-sources.jar: carpetaddonsnotfound/mixins/LivingEntity_DropAllXpOnPlayerDeathMixin.java <==
package carpetaddonsnotfound.mixins;

import carpetaddonsnotfound.CarpetAddonsNotFoundSettings;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_3218;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Redirect;

@Mixin(class_1309.class)
public abstract class LivingEntity_DropAllXpOnPlayerDeathMixin {
  @Redirect(
          //#if MC>12103
          method = "dropExperience",
          //#else
          //$$ method = "dropXp",
          //#endif
          at = @At(
                  value = "INVOKE",
                  //#if MC>12103
                  target = "Lnet/minecraft/entity/LivingEntity;getExperienceToDrop(Lnet/minecraft/server/world/ServerWorld;Lnet/minecraft/entity/Entity;)I"
                  //#elseif MC>12006
                  //$$ target = "Lnet/minecraft/entity/LivingEntity;getXpToDrop(Lnet/minecraft/server/world/ServerWorld;Lnet/minecraft/entity/Entity;)I"
                  //#elseif MC>11802
                  //$$ target = "Lnet/minecraft/entity/LivingEntity;getXpToDrop()I"
                  //#else
                  //$$ target = "Lnet/minecraft/entity/LivingEntity;getXpToDrop(Lnet/minecraft/entity/player/PlayerEntity;)I"
                  //#endif
  ))
  private int onDropXp(
          //#if MC>12006
          class_1309 instance,
          class_3218 world,
          class_1297 attacker
          //#elseif MC>11802
          //$$ LivingEntity instance
          //#else
          //$$ LivingEntity instance,
          //$$ PlayerEntity attackingPlayer
          //#endif
                      ) {
    if (!CarpetAddonsNotFoundSettings.dropAllXpOnPlayerDeath ||
        !((class_1309) instance instanceof class_1657 player)) {
      //#if MC>12103
      return instance.method_59923(world, attacker);
      //#elseif MC>12006
      //$$ return instance.getXpToDrop(world, attacker);
      //#elseif MC>11802
      //$$ return instance.getXpToDrop();
      //#else
      //$$ return ((LivingEntityInvokerMixin) instance).invokeGetXpToDrop(attackingPlayer);
      //#endif
    }

    double totalExperience = calculateTotalExperience(player);
    return (int)Math.ceil(totalExperience);
  }

  // This exists because just using the totalExperience field doesn't work if levels have been set separately to xp points
  @Unique
  private double calculateTotalExperience(class_1657 player) {
    double xpLevel = player.field_7520 + player.field_7510;
    if (xpLevel < 17) {
      return Math.pow(xpLevel, 2) + 6 * xpLevel;
    }

    if (xpLevel < 32) {
      return 2.5 * Math.pow(xpLevel, 2) - 40.5 * xpLevel + 360;
    }

    return 4.5 * Math.pow(xpLevel, 2) - 162.5 * xpLevel + 2220;
  }
}
